/* chil2a.c -- converts Chilbolton-format radar data to ascii
   A multitude of processing options are available

   Copyright (C) 2000 Robin Hogan
*/
#include <stdio.h>
#include "chil.h"

#define MIN(a, b) ((a) < (b) ? (a) : (b))

int
main(int argc, char **argv)
{
  FILE *afile = NULL;
  int status, iarg = 0;
  ChilboltonData data;
  ChilboltonFile *chil_file = &(data.chil_file);
  RasterInfo *raster_info = &(data.raster_info);
  Ray *ray = &(data.ray);
  char *tmp_name, *id = "";
  char aname[20];
  char cloud_data;
  unsigned short hour, minute, second;
  unsigned short startgate, endgate, nfields = 1;

  status = CHILInit(argc, argv, &data);
  if (argc < 2)
    {
      CHILUsage(&data);
      CHILOptions(&data);
      exit(0);
    }


  /* Look for ascii-related arguments */
  
  while (++iarg < argc) {
    if (strcmp(argv[iarg],"-id") == 0) {
      if (argc > iarg+1) {
	id = argv[++iarg];
      }
      else {
	CHILPrintf("Error: id not specified\n");
	exit(1);
      }
    }
  }

  if (status == CHIL_OK)
    {
      while (1)
	{
	  int i;
	  int failures = 0, maxfailures = 10;

	  while ((status = CHILFetchRay(&data)) != CHIL_OK)
	    {
	      if (status != CHIL_ENDOFBLOCK) {
		break;
	      }
	      if ((++failures) == maxfailures)
		{
		  CHILPrintf("Aborting: %d failed calls to CHILFetchRay - empty scans?\n");
		  break;
		}
	    }
	  if (status != CHIL_OK)
	    {
	      break;
	    }

	  cloud_data = raster_info->cloud_data;
	  if (cloud_data)
	    {
	      sprintf(aname,"%04hu%02hu%02hu_%d%s.dat",
		      CHILTrueYear(ray->year),
		      ray->month, ray->day, (int) raster_info->frequency,
		      id);
	    }
	  else
	    {
	      sprintf(aname,"%04hur%03hus%02hu%s.dat", raster_info->file,
		      raster_info->raster, chil_file->iscan+1, id);
	    }
	  afile = fopen(aname, "w");
	  if (!data.quiet)
	    CHILPrintf("Writing %s\n", aname);
	  nfields = MIN(ray->ntotal, data.maxfields);
	  startgate = ray->startgate;
	  endgate = ray->endgate;

	  if (cloud_data) {
	    fprintf(afile, "%%File: %04hu\n%%Raster: %hu\n",
		    raster_info->file, raster_info->raster);
	  }
	  else {
	    fprintf(afile, "%%File: %04hu\n%%Raster: %hu\n%%Scan: %hu\n",
		    raster_info->file, raster_info->raster, chil_file->iscan+1);
	  }
	  fprintf(afile, "%%Date: %04hu%02hu%02hu\n",
		  CHILTrueYear(ray->year), ray->month, ray->day);

	  if (cloud_data || ray->utc != 0) {
	    hour = (unsigned short) ray->utc;
	    minute = (unsigned short) (60.0 * (ray->utc - (float) hour));
	    second = (unsigned short) (3600.0 * ((ray->utc - (float) hour)
						 - minute/60.0));
	  }
	  else {
	    hour = raster_info->hour;
	    minute = raster_info->minute;
	    second = raster_info->second;
	  }
	  fprintf(afile, "%%Time: %02hu:%02hu:%02hu UTC\n",
		  hour, minute, second);

	  fprintf(afile, "%%System: %s\n", CHILSystemName(raster_info,
							  NULL));
	  fprintf(afile, "%%Frequency: %g GHz\n", raster_info->frequency);
	  fprintf(afile, "%%Latitude: %g degrees_north\n"
		  "%%Longitude: %g degrees_east\n%%Altitude: %g m\n",
		  CHILLatitude(raster_info),
		  CHILLongitude(raster_info),
		  CHILAltitude(raster_info));
	  fprintf(afile, "%%Gates: %u\n", endgate - startgate);
	  fprintf(afile, "%%Columns: %u\n", 2+(!cloud_data)*2+nfields);
	  fprintf(afile, "%%ScanType: %s\n", CHILScanType(raster_info, NULL));
	  fprintf(afile, "%%MissingValue: %g\n", chil_no_signal_value);
	  fprintf(afile, "%%SoftwareVersion: " CHIL_VERSION "\n");
	  fprintf(afile, "%%Options: ");
	  tmp_name = CHILGetOptions(&data);
	  fprintf(afile, tmp_name);
	  CHILFree(tmp_name);

	  if (cloud_data)
	    {
	      fprintf(afile, "\n%%Variables: time range");
	    }
	  else
	    {
	      fprintf(afile, "\n%%Variables: time azimuth elevation range");
	    }

	  for (i = 0; i < nfields; i++)
	    {
	      fprintf(afile, " %s", CHILFieldName(ray, i, NULL));
	    }

	  if (cloud_data)
	    {
	      fprintf(afile, "\n%%Units: decimal_hours_UTC km");
	    }
	  else
	    {
	      fprintf(afile, "\n%%Units: decimal_hours_UTC degrees_from_west degrees km");
	    }

	  for (i = 0; i < nfields; i++)
	    {
	      fprintf(afile, " %s", CHILFieldUnits(ray, i, NULL));
	    }

	  do
	    {
	      if (cloud_data)
		{
		  int j;
		  for (j = startgate; j < endgate; j++)
		    {
		      int i;
		      fprintf(afile, "\n%7.4f %7.3f", ray->utc,
			      ray->range[j]+0.001*ray->range_offset);
		      for (i = 0; i < nfields; i++)
			{
			  fprintf(afile, " %9.4f", *(ray->data[i]+j));
			}
		    }
		}
	      else
		{
		  int j;
		  for (j = startgate; j < endgate; j++)
		    {
		      int i;
		      fprintf(afile, "\n%7.4f %8.3f %8.3f %7.3f", ray->utc,
			      ray->azimuth, ray->elevation,
			      ray->range[j]+0.001*ray->range_offset);
		      for (i = 0; i < nfields; i++)
			{
			  fprintf(afile, " %9.4f", *(ray->data[i]+j));
			}
		    }

		}
	    }
	  while ((status = CHILFetchRay(&data)) == CHIL_OK);

	  fprintf(afile, "\n");

	  fclose(afile);
	  if (status == CHIL_ENDOFDATA)
	    {
	      break;
	    }
	}
      CHILEnd(&data);
    }
  else {
    CHILPrintf("Run with no options for more information\n");
    exit(1);
  }
  exit(0);
}
